@@ -19,8 +19,8 @@ Modified by Jonathan Larmour <jifl@jifvik.org>.
 Copyright of the modifications 2002.
 Modified by Richard van der Hoff <richard@rvanderhoff.org.uk>
 Copyright of the modifications 2002.
-Enhanced and modified by Matthias Andree <matthias.andree@web.de>.
-Copyright of the modifications 2000 - 2003.
+Enhanced and modified by Matthias Andree <matthias.andree@gmx.de>.
+Copyright of the modifications 2000 - 2006.
 
 See file COPYING for restrictions on the use of this software.
 */
@@ -932,7 +932,7 @@ getgroup(const struct server *current_server,
 				tmp, crosspostlimit);
 		    } else if (lstat(t = lookup(fields[h_mid]), &st) == 0) {
 			killed++;
-			ln_log(LNLOG_SINFO, 4, "%s: killed %lu (%s), already fetched before",
+			ln_log(LNLOG_SDEBUG, 4, "%s: killed %lu (%s), already fetched before",
 				g->name, art, fields[h_mid]);
 		    } else {
 			stufftoget[outstanding] = art;
@@ -2248,6 +2248,30 @@ error_refetch(const char *e) {
     ln_log(LNLOG_SERR, LNLOG_CTOP, "REASON: %s", e);
 }
 
+/** re-add non-expiring groups to active */
+static void
+addnonexpiring(void)
+{
+    struct stringlist *t, *l = get_grouplist();
+    int expdays;
+
+    for (t=l; t != NULL; t = t->next) {
+	char *x = t->string;
+
+	if ((expdays = lookup_expiredays(x)) >= 0) {
+	    if (expdays == 0 || !(expdays = lookup_expire(x)))
+		expdays = expire;
+	} else {
+	    expdays = -1;
+	}
+
+	if (expdays == -1) {
+	    insertgroup(x, 0, 0, 0);
+	}
+    }
+    freelist(l);
+}
+
 int
 main(int argc, char **argv)
 {
@@ -2322,8 +2346,9 @@ main(int argc, char **argv)
 	exit(1);
     }
 
-    syslog(LOG_DEBUG, "leafnode %s: verbosity level is %d, debugmode is %d",
-	    version, verbose, debugmode);
+    if (debugmode)
+	syslog(LOG_DEBUG, "leafnode %s: verbosity level is %d, debugmode is %d",
+		version, verbose, debugmode);
     if (verbose || debugmode) {
 	printf("leafnode %s: verbosity level is %d, debugmode is %d\n",
 		version, verbose, debugmode);
@@ -2347,6 +2372,7 @@ main(int argc, char **argv)
     if (!postonly) {
 	readactive();
 	if (!active) {
+	    addnonexpiring();
 	    fakeactive(); /* we need proper lowwater/highwater marks
 			     for the groups that exist */
 	    ln_log(LNLOG_SNOTICE, LNLOG_CTOP,
@@ -2395,6 +2421,7 @@ main(int argc, char **argv)
 	active = NULL;
 	activesize = 0;
 	if (killactiveread()) exit(1);
+	addnonexpiring();
     }
 
     if (mysigact(SIGINT, 0, sig_int, SIGALRM) != 0)
