@@ -14,7 +14,7 @@ Copyright of the modifications 1998.
 Modified by Cornelius Krasel <krasel@wpxx02.toxi.uni-wuerzburg.de>
 and Kazushi (Jam) Marukawa <jam@pobox.com>.
 Copyright of the modifications 1998, 1999.
-Modified by Matthias Andree <matthias.andree@web.de>
+Modified by Matthias Andree <matthias.andree@gmx.de>
 Copyright of the modifications 2000 - 2002.
 Modified by Ralf Wildenhues <ralf.wildenhues@gmx.de>
 Copyright of the modifications 2002.
@@ -173,7 +173,7 @@ buildpseudoart(const char *grp)
 	    "\n"
 	    "If you do not understand this, please talk to your newsmaster.\n"
 	    "\n"
-	    "Leafnode can be found at\n" "\thttp://leafnode.sourceforge.net/\n\n");
+	    "Leafnode can be found at\n" "\thttp://www.leafnode.org/\n\n");
 
     if (ferror(f)) {
 	(void)fclose(f);
@@ -225,6 +225,12 @@ is_pseudogroup(/*@null@*/ const struct newsgroup *g)
     return 0;
 }
 
+/*
+ * XXX FIXME: an option is needed if the administrator
+ * locks the interesting.group permissions in order to have a fixed set
+ * of subscriptions that are always available, or if the subscription
+ * is handled outside leafnode.
+ */
 static void
 markinterest(const char *ng)
 {
@@ -235,23 +241,52 @@ markinterest(const char *ng)
     FILE *f;
     char s[SIZE_s+1];
 
+    /* OK, the user may be setting the file to root owned with group
+     * write permission. This is a problem when trying to set the ctime
+     * without setting the mtime, because we would need privileges to do
+     * that. Instead, we'll bump both times and warn the user. */
+
     err = 0;
     (void)xsnprintf(s, SIZE_s, "%s/interesting.groups/%s", spooldir, ng);
     if (stat(s, &st) == 0) {
+	const char *touched = "ctime";
+	/* group was read before: keep mtime, but bump up ctime */
 	now = time(NULL);
 	buf.actime = (now < st.st_atime) ? st.st_atime : now;
 	/* now < update may happen through HW failures */
 	buf.modtime = st.st_mtime;
 	if (utime(s, &buf)) {
-	    err = 1;
+	    int oe = errno;
+	    /* cannot set time - the file's user-id must be the NEWS_USER's
+	     * 2nd best - we'll try to bump both times, but this means
+	     * timeout_short applies, so warn the user. */
+	    if (utime(s, NULL)) {
+		/* ok, didn't work either. complain and give up.
+		 * We used to fall through to the code below, but that
+		 * may truncate the file and thus lose delaybody article
+		 * numbers. */
+		syslog(LOG_ERR, "Error: cannot set ctime/mtime timestamp of %s: %s.",
+			s, strerror(errno));
+		return;
+	    } else {
+		/* complain that timeout_short applies, so that the user
+		 * is not surprised by premature group unsubscription. */
+		touched = "ctime and mtime";
+		if (timeout_short < timeout_long) {
+		    syslog(LOG_WARNING, "Warning: cannot set the ctime timestamp of %s without resetting mtime (%s). "
+			    "This means that timeout_short will apply rather than timeout_long. "
+			    "Either reset file ownership to " NEWS_USER " or set timeout_short and timeout_long to the same value to suppress this warning.",
+			    s, strerror(oe));
+		}
+	    }
 	}
-	if (debugmode)
-	    syslog(LOG_DEBUG, "markinterest: %s touched", ng);
+	if (debugmode && !err)
+	    syslog(LOG_DEBUG, "markinterest: %s touched %s", ng, touched);
     } else {
 	err = 1;
     }
     if (err) {
-	f = fopen(s, "w");
+	f = fopen(s, "w"); /* truncating sets ctime and mtime */
 	if (f == NULL || fclose(f) == EOF) {
 	    syslog(LOG_ERR, "Could not create %s: %m", s);
 	} else {
